@@ -4,6 +4,238 @@
 
 //#define DEBUG
 
+#define PACKAGE_NAME "nss"
+
+#if PY_VERSION_HEX <  0x02070000
+// As part of the Python 2 to Python 3 conversion we need at least
+// version 2.7 because 2.7 shares API's with 3.x
+#error "Python version must be at least 2.7"
+#endif
+
+#define PyNone_Check(x) ((x) == Py_None)
+
+/************************ BEGIN PYTHON VERSION COMPARABILITY ******************/
+
+#if PY_MAJOR_VERSION >= 3
+
+/******************************************************************************
+ *                                  Python 3                                  *
+ ******************************************************************************/
+
+
+#define IS_PY3K
+
+#define MOD_ERROR_VAL NULL
+#define MOD_SUCCESS_VAL(val) val
+#define MOD_INIT(name) PyMODINIT_FUNC PyInit_##name(void)
+
+#define PyInteger_Check(obj)  PyLong_Check(obj)
+#define PyBaseString_Check(obj) PyUnicode_Check(obj)
+
+static inline PyObject *
+PyBaseString_UTF8(PyObject *obj, char *name)
+{
+    if (obj == NULL) {
+        return PyUnicode_FromString("<NULL>");
+    }
+
+    if (PyUnicode_Check(obj)) {
+        return PyUnicode_AsUTF8String(obj);
+    }
+
+    if (name) {
+        PyErr_Format(PyExc_TypeError, "%s must be a string, not %.200s",
+                     name, Py_TYPE(obj)->tp_name);
+        return NULL;
+    } else {
+        PyErr_Format(PyExc_TypeError, "must be a string, not %.200s",
+                     Py_TYPE(obj)->tp_name);
+        return NULL;
+    }
+}
+
+/* Returns new reference */
+static inline PyObject *
+PyUnicode_from_basestring(PyObject *obj)
+{
+    if (PyUnicode_Check(obj)) {
+        Py_INCREF(obj);
+        return obj;
+    }
+
+    PyErr_Format(PyExc_TypeError, "must be string, not %.200s",
+                 Py_TYPE(obj)->tp_name);
+
+    return NULL;
+}
+
+/* Returns new reference */
+static inline PyObject *
+PyObject_String(PyObject *obj)
+{
+    return PyObject_Str(obj);
+}
+
+static inline int
+UnicodeOrNoneConvert(PyObject *obj, PyObject **param)
+{
+    if (!obj) {
+        *param = NULL;
+        return 1;
+    }
+
+    if (PyNone_Check(obj)) {
+        *param = NULL;
+        return 1;
+    }
+
+    if (PyUnicode_Check(obj)) {
+        Py_INCREF(obj);
+        *param = obj;
+        return 1;
+    }
+
+    PyErr_Format(PyExc_TypeError, "must be a string or None, not %.200s",
+                 Py_TYPE(obj)->tp_name);
+
+    return 0;
+}
+
+#else  /* PY_MAJOR_VERSION < 3 */
+
+/******************************************************************************
+ *                                  Python 2                                  *
+ ******************************************************************************/
+
+#include "bytesobject.h"
+
+#define MOD_ERROR_VAL
+#define MOD_SUCCESS_VAL(val)
+#define MOD_INIT(name) void init##name(void)
+
+#define PyInteger_Check(obj) (PyInt_Check(obj) || PyLong_Check(obj))
+#define PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
+
+/* returns new reference or NULL on error */
+static inline PyObject *
+PyBaseString_UTF8(PyObject *obj, char *name)
+{
+    if (obj == NULL) {
+        return PyUnicode_FromString("<NULL>");
+    }
+
+    if (PyString_Check(obj)) {
+        Py_INCREF(obj);
+        return obj;
+    }
+
+    if (PyUnicode_Check(obj)) {
+        return PyUnicode_AsUTF8String(obj);
+    }
+
+    if (name) {
+        PyErr_Format(PyExc_TypeError, "%s must be a string, not %.200s",
+                     name, Py_TYPE(obj)->tp_name);
+        return NULL;
+    } else {
+        PyErr_Format(PyExc_TypeError, "must be a string, not %.200s",
+                     Py_TYPE(obj)->tp_name);
+        return NULL;
+    }
+}
+
+/* Returns new reference */
+static inline PyObject *
+PyUnicode_from_basestring(PyObject *obj)
+{
+    if (PyUnicode_Check(obj)) {
+        Py_INCREF(obj);
+        return obj;
+    }
+
+    if (PyString_Check(obj)) {
+        return PyUnicode_FromString(PyString_AS_STRING(obj)); /* decodes utf-8 */
+    }
+
+    PyErr_Format(PyExc_TypeError, "must be string, not %.200s",
+                 Py_TYPE(obj)->tp_name);
+
+    return NULL;
+}
+
+/* Returns new reference */
+static inline PyObject *
+PyObject_String(PyObject *obj)
+{
+    return PyObject_Unicode(obj);
+}
+
+/*
+ * Py2 does not have file system codecs, emulate PyUnicode_FSConverter()
+ * to the best extent possible, convert the unicode to encoded UTF-8.
+ * UTF-8 is not the same as file system encoding but it' the best we can
+ * do. Caller must DECREF the returned arg.
+ */
+static inline int
+PyUnicode_FSConverter(PyObject* arg, void* addr)
+{
+    PyObject *output = NULL;
+
+    if (PyString_Check(arg)) {
+        output = arg;
+        Py_INCREF(output);
+    } else if (PyUnicode_Check(arg)) {
+        if ((output = PyUnicode_AsUTF8String(arg)) == NULL) {
+            return 0;
+        }
+    } else {
+        PyErr_Format(PyExc_TypeError, "must be str or unicode, not %.50s",
+                     Py_TYPE(arg)->tp_name);
+        return 0;
+    }
+
+    *(PyObject**)addr = output;
+    return 1;
+}
+
+static inline int
+UnicodeOrNoneConvert(PyObject *obj, PyObject **param)
+{
+    if (!obj) {
+        *param = NULL;
+        return 1;
+    }
+
+    if (PyNone_Check(obj)) {
+        *param = NULL;
+        return 1;
+    }
+
+    if (PyString_Check(obj)) {
+        if ((*param = PyUnicode_FromString(PyString_AS_STRING(obj))) == NULL) {
+            return 0;
+        }
+        return 1;
+    }
+
+    if (PyUnicode_Check(obj)) {
+        Py_INCREF(obj);
+        *param = obj;
+        return 1;
+    }
+
+    PyErr_Format(PyExc_TypeError, "must be a string or None, not %.200s",
+                 Py_TYPE(obj)->tp_name);
+
+    return 0;
+}
+
+
+#endif  /* PY_MAJOR_VERSION */
+
+/************************* END PYTHON VERSION COMPARABILITY *******************/
+
+
 typedef PyObject *(*format_lines_func)(PyObject *self, PyObject *args, PyObject *kwds);
 
 typedef enum RepresentationKindEnum {
@@ -153,44 +385,133 @@ do {                                            \
 
 /******************************************************************************/
 
+/*
+ * PyBytes_From_BaseString
+ *
+ * param obj: Python string object
+ * param encoding: Encoding to use, if NULL then utf-8
+ *
+ * Return: PyBytes object if successful which must be released with Py_DECREF.
+ *         If unsuccessful NULL is returned and an exception will have been set.
+ *
+ * Convert a Python string to encoded bytes.
+ *
+ * Python2: The string may be either a PyUnicode or a PyString object.
+ * If it's a PyString object it is assumed the value is already encoded
+ * and a copy is returned.
+ *
+ * Python3: The string must be a PyUnicode object.
+ *
+ * Example:
+ *
+ * PyObject *encoded = NULL;
+ *
+ * if ((encoded = PyBytes_From_BaseString(obj, NULL)) == NULL) {
+ *     return NULL;
+ * }
+ * some_c_function(PyBytes_AS_STRING(encoded));
+ * Py_DECREF(encoded);
+ */
+
+static inline PyObject *PyBytes_From_BaseString(PyObject *obj, const char *encoding)
+{
+    if (!PyBaseString_Check(obj)) {
+        PyErr_Format(PyExc_TypeError, "must be string, not %.50s",
+                     Py_TYPE(obj)->tp_name);
+        return NULL;
+    }
+
+#if PY_MAJOR_VERSION < 3
+    if (PyString_Check(obj)) {
+        return PyBytes_FromString(PyString_AS_STRING(obj));
+    }
+#endif
+
+    return PyUnicode_AsEncodedString(obj, encoding ? encoding : "utf-8", NULL);
+}
+
+static inline void PyUnicode_ConcatAndDel(PyObject** left, PyObject* right) {
+    PyObject *tmp;
+
+    tmp = PyUnicode_Concat(*left, right);
+    Py_XDECREF(*left);
+    Py_XDECREF(right);
+    *left = tmp;
+}
+
+static inline PyObject *
+PyUnicode_Lower(PyObject *obj)
+{
+    PyObject *py_unicode = NULL;
+    PyObject *py_lower = NULL;
+
+    if ((py_unicode = PyUnicode_from_basestring(obj)) == NULL) {
+        return NULL;
+    }
+
+    if ((py_lower = PyObject_CallMethod(obj, "lower", NULL)) == NULL) {
+        Py_DECREF(py_unicode);
+        return NULL;
+    }
+
+    Py_DECREF(py_unicode);
+    return py_lower;
+}
+
+
+/******************************************************************************/
+
+#define RETURN_COMPARE_RESULT(op, cmp_result)   \
+{                                               \
+    switch(op) {                                \
+    case Py_LT:                                 \
+        if (cmp_result < 0)                     \
+            Py_RETURN_TRUE;                     \
+        else                                    \
+            Py_RETURN_FALSE;                    \
+    case Py_LE:                                 \
+        if (cmp_result <= 0)                    \
+            Py_RETURN_TRUE;                     \
+        else                                    \
+            Py_RETURN_FALSE;                    \
+    case Py_EQ:                                 \
+        if (cmp_result == 0)                    \
+            Py_RETURN_TRUE;                     \
+        else                                    \
+            Py_RETURN_FALSE;                    \
+    case Py_NE:                                 \
+        if (cmp_result != 0)                    \
+            Py_RETURN_TRUE;                     \
+        else                                    \
+            Py_RETURN_FALSE;                    \
+    case Py_GT:                                 \
+        if (cmp_result > 0)                     \
+            Py_RETURN_TRUE;                     \
+        else                                    \
+            Py_RETURN_FALSE;                    \
+    case Py_GE:                                 \
+        if (cmp_result >= 0)                    \
+            Py_RETURN_TRUE;                     \
+        else                                    \
+            Py_RETURN_FALSE;                    \
+    }                                           \
+                                                \
+    Py_RETURN_FALSE;                            \
+}
+
+#define Py_RETURN_BOOL(condition) {if (condition) Py_RETURN_TRUE; else Py_RETURN_FALSE;}
+
 // Gettext
 #ifndef _
 #define _(s) s
 #endif
 
-#if (PY_VERSION_HEX < 0x02050000)
-typedef int Py_ssize_t;
-#define PY_SSIZE_T_MAX INT_MAX
-#define PY_SSIZE_T_MIN INT_MIN
-#define PyInt_FromSsize_t PyInt_FromLong
-#define PyNumber_AsSsize_t(ob, exc) PyInt_AsLong(ob)
-#define PyIndex_Check(ob) PyInt_Check(ob)
-typedef Py_ssize_t (*readbufferproc)(PyObject *, Py_ssize_t, void **);
-typedef Py_ssize_t (*writebufferproc)(PyObject *, Py_ssize_t, void **);
-typedef Py_ssize_t (*segcountproc)(PyObject *, Py_ssize_t *);
-typedef Py_ssize_t (*charbufferproc)(PyObject *, Py_ssize_t, char **);
-typedef Py_ssize_t (*lenfunc)(PyObject *);
-typedef PyObject *(*ssizeargfunc)(PyObject *, Py_ssize_t);
-typedef PyObject *(*ssizessizeargfunc)(PyObject *, Py_ssize_t, Py_ssize_t);
-#endif
-
-#if (PY_VERSION_HEX < 0x02060000)
-#define Py_TYPE(ob) (((PyObject*)(ob))->ob_type)
-#define PyVarObject_HEAD_INIT(type, size) \
-	PyObject_HEAD_INIT(type) size,
-#define PyImport_ImportModuleNoBlock PyImport_ImportModule
-#define PyLong_FromSsize_t PyInt_FromLong
-#define Py_TPFLAGS_HAVE_NEWBUFFER 0
-#endif
-
-#define PyNone_Check(x) ((x) == Py_None)
-
 #define CALL_BASE(type, func, ...) (type)->tp_base->tp_##func(__VA_ARGS__)
 
 #define TYPE_READY(type)                                                \
 {                                                                       \
     if (PyType_Ready(&type) < 0)                                        \
-        return;                                                         \
+        return MOD_ERROR_VAL;                                           \
     Py_INCREF(&type);                                                   \
     PyModule_AddObject(m, rindex(type.tp_name, '.')+1, (PyObject *)&type); \
 }
@@ -203,14 +524,14 @@ typedef PyObject *(*ssizessizeargfunc)(PyObject *, Py_ssize_t, Py_ssize_t);
     if ((dict = PyModule_GetDict(m)) == NULL) {                         \
         PyErr_Format(PyExc_SystemError, "module '%s' has no __dict__",  \
                      PyModule_GetName(m));                              \
-        return;                                                         \
+        return MOD_ERROR_VAL;                                           \
     }                                                                   \
     if (PyDict_GetItemString(dict, #c)) {                               \
         PyErr_Format(PyExc_SystemError, "module '%s' already contains %s", \
                          PyModule_GetName(m), #c);                      \
-        return;                                                         \
+        return MOD_ERROR_VAL;                                           \
     }                                                                   \
-    if (PyModule_AddIntConstant(m, #c, c) < 0) return;                  \
+    if (PyModule_AddIntConstant(m, #c, c) < 0) return MOD_ERROR_VAL;    \
 }
 
 #define AddIntConstantName(name, c)                                     \
@@ -221,14 +542,14 @@ typedef PyObject *(*ssizessizeargfunc)(PyObject *, Py_ssize_t, Py_ssize_t);
     if ((dict = PyModule_GetDict(m)) == NULL) {                         \
         PyErr_Format(PyExc_SystemError, "module '%s' has no __dict__",  \
                      PyModule_GetName(m));                              \
-        return;                                                         \
+        return MOD_ERROR_VAL;                                           \
     }                                                                   \
     if (PyDict_GetItemString(dict, #c)) {                               \
         PyErr_Format(PyExc_SystemError, "module '%s' already contains %s", \
                          PyModule_GetName(m), #c);                      \
-        return;                                                         \
+        return MOD_ERROR_VAL;                                           \
     }                                                                   \
-    if (PyModule_AddIntConstant(m, #name, c) < 0) return;               \
+    if (PyModule_AddIntConstant(m, #name, c) < 0) return MOD_ERROR_VAL; \
 }
 
 #ifdef DEBUG
@@ -236,7 +557,7 @@ typedef PyObject *(*ssizessizeargfunc)(PyObject *, Py_ssize_t, Py_ssize_t);
 #define DumpRefCount(x)                                                 \
 {                                                                       \
     PyObject *_obj = (PyObject *) (x);                                  \
-    printf("<%s object at %p refcnt=%d>\n", Py_TYPE(_obj)->tp_name, _obj, _obj->ob_refcnt); \
+    printf("<%s object at %p refcnt=%zd>\n", Py_TYPE(_obj)->tp_name, _obj, _obj->ob_refcnt); \
 }
 
 
@@ -253,7 +574,7 @@ typedef PyObject *(*ssizessizeargfunc)(PyObject *, Py_ssize_t, Py_ssize_t);
     if (_obj) {                                                         \
         name = Py_TYPE(_obj)->tp_name;                                  \
     }                                                                   \
-    printf("%s (Enter): <%s object at %p refcnt=%d>\n",                 \
+    printf("%s (Enter): <%s object at %p refcnt=%zd>\n",                \
            __FUNCTION__, name, _obj, _obj ? _obj->ob_refcnt : -9999);   \
 }
 
@@ -265,7 +586,7 @@ typedef PyObject *(*ssizessizeargfunc)(PyObject *, Py_ssize_t, Py_ssize_t);
     if (_obj) {                                                         \
         name = Py_TYPE(_obj)->tp_name;                                  \
     }                                                                   \
-    printf("%s (Leave): <%s object at %p refcnt=%d>\n",                 \
+    printf("%s (Leave): <%s object at %p refcnt=%zd>\n",                \
            __FUNCTION__, name, _obj, _obj ? _obj->ob_refcnt : -9999);   \
 }
 
@@ -287,7 +608,7 @@ typedef PyObject *(*ssizessizeargfunc)(PyObject *, Py_ssize_t, Py_ssize_t);
     if (_obj) {                                                         \
         name = Py_TYPE(_obj)->tp_name;                                  \
     }                                                                   \
-    printf("%s: returns <%s object at %p refcnt=%d>\n",                 \
+    printf("%s: returns <%s object at %p refcnt=%zd>\n",                \
            __FUNCTION__, name, _obj, _obj ? _obj->ob_refcnt : -9999);   \
 }
 
