@@ -2,6 +2,11 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#ifndef NSS_ERROR_MODULE_H
+#define NSS_ERROR_MODULE_H
+
+#define NSS_ERROR_MODULE_NAME "error"
+
 #include <stdbool.h>
 
 /* NSPR header files */
@@ -18,7 +23,7 @@ typedef struct {
 typedef struct {
     PyObject     *nspr_exception;
     PyObject     *(*set_nspr_error)(const char *format, ...) __attribute__ ((format (printf, 1, 2)));
-    PyObject     *(*set_cert_verify_error)(unsigned int usages, PyObject * log, const char *format, ...)  __attribute__ ((format (printf, 3, 4)));
+    PyObject     *(*set_cert_verify_error)(unsigned long usages, PyObject * log, const char *format, ...)  __attribute__ ((format (printf, 3, 4)));
     PyObject     *(*tuple_str)(PyObject *tuple);
     const NSPRErrorDesc *(*lookup_nspr_error)(PRErrorCode num);
 } PyNSPR_ERROR_C_API_Type;
@@ -49,13 +54,13 @@ import_nspr_error_c_api(void)
         return -1;
     }
 
-    if (!(PyCObject_Check(c_api_object))) {
+    if (!(PyCapsule_CheckExact(c_api_object))) {
         Py_DECREF(c_api_object);
         Py_DECREF(module);
         return -1;
     }
 
-    if ((api = PyCObject_AsVoidPtr(c_api_object)) == NULL) {
+    if ((api = PyCapsule_GetPointer(c_api_object, "_C_API")) == NULL) {
         Py_DECREF(c_api_object);
         Py_DECREF(module);
         return -1;
@@ -68,3 +73,4 @@ import_nspr_error_c_api(void)
 }
 
 #endif /* NSS_ERROR_MODULE */
+#endif /* NSS_ERROR_MODULE_H */
