@@ -1,5 +1,5 @@
-#!/usr/bin/python
-
+from __future__ import print_function
+from __future__ import absolute_import
 import sys
 import os
 import unittest
